@@ -54,8 +54,27 @@ class Combined(models.Model):
         enabled = dbsettings.BooleanValue()
     settings = TestSettings() + settings()
 
+# For registration testing
+class ClashSettings1(dbsettings.Group):
+    clash1 = dbsettings.BooleanValue()
+
+class ClashSettings2(dbsettings.Group):
+    clash2 = dbsettings.BooleanValue()
+
+class ClashSettings1_2(dbsettings.Group):
+    clash1 = dbsettings.IntegerValue()
+    clash2 = dbsettings.IntegerValue()
+
+module_clash1 = ClashSettings1()
+
+class ModelClash(models.Model):
+    settings = ClashSettings1_2()
+
+module_clash2 = ClashSettings2()
+
+
 class SettingsTestCase(test.TestCase):
-    urls = 'dbsettings.urls'
+    urls = 'dbsettings.tests.test_urls'
 
     def setUp(self):
         # Standard test fixtures don't update the in-memory cache.
@@ -228,6 +247,20 @@ class SettingsTestCase(test.TestCase):
         self.assert_('can_edit_populated_settings' in dict(Populated._meta.permissions))
         self.assert_('can_edit_unpopulated_settings' in dict(Unpopulated._meta.permissions))
 
+    def assertCorrectSetting(self, value_class, *key):
+        from dbsettings import loading
+        setting = loading.get_setting(*key)
+        self.assertEqual(key, setting.key)  # Check if setting is registered with proper key
+        self.assertTrue(isinstance(setting, value_class))
+
+    def test_registration(self):
+        "Module and class settings can be mixed up"
+        from dbsettings import BooleanValue, IntegerValue
+        self.assertCorrectSetting(BooleanValue, 'dbsettings.tests.tests', '', 'clash1')
+        self.assertCorrectSetting(IntegerValue, 'dbsettings.tests.tests', 'ModelClash', 'clash1')
+        self.assertCorrectSetting(IntegerValue, 'dbsettings.tests.tests', 'ModelClash', 'clash2')
+        self.assertCorrectSetting(BooleanValue, 'dbsettings.tests.tests', '', 'clash2')
+
     def test_forms(self):
         "Forms should display only the appropriate settings"
         from django.contrib.auth.models import User, Permission
@@ -302,3 +335,11 @@ class SettingsTestCase(test.TestCase):
         self.assertEqual(Editable.settings.date, datetime.date(2012, 6, 28))
         self.assertEqual(Editable.settings.time, datetime.time(16, 37, 45))
         self.assertEqual(Editable.settings.datetime, datetime.datetime(2012, 6, 28, 16, 37, 45))
+
+        # Check if module / class settings show properly
+        #perm = Permission.objects.get(codename='can_edit__settings')  # module-level settings
+        #user.user_permissions.add(perm)
+
+        #response = self.client.get(site_form)
+        #self.assertTemplateUsed(response, 'dbsettings/site_settings.html')
+        #self.assertEqual(len(response.context[0]['form'].fields), 14)
